%STOCH_DIFFPOST legacy a posteriori estimated error distribution
% This implements the Q2-based error approximation scheme 
% for the stochastic Galerkin FE solution
%   SIFISS scriptfile: DJS; 5 July 2017.
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

% set the method for enriching the polynomial space
  pm_choice=default('p/M/pM enrichment of the polynomial space 1/2/3? (default p-enrichment)',1);
  
  if qmethod ==1,
     fprintf('FAST a posteriori error estimation for Q1 element \n') 
     [hx,hy,eex] = edgegen(xy,ev);
% call stoch_diffpost_q1_ypx for the V_{YP}-estimator
     [yp_err_sq_el,yp_elerr,fe,ae] = stoch_diffpost_q1_ypx(xy,ev,ebound,x_gal,...
         eex,hx,hy,indset,P,norv,noarv,KL_DATA,0);
% call stoch_diffpost_q1_yqx for the V_{YQ}-estimator
     [yq_err_sq_el,yq_elerr,fe,ae] = stoch_diffpost_q1_yqx(xy,ev,ebound,x_gal,...
         eex,hx,hy,indset,polyd,norv,noarv,KL_DATA,pm_choice,1);
% call stoch_diffpost_q1_xqx for the V_{XQ}-estimator
% the error estimate is given by xq_err_est
     [xq_err_gal,xq_err_est,xq_err_sq_el] = stoch_diffpost_q1_xqx(xy,ev,bound,x_gal,...
         indset,polyd,norv,noarv,KL_DATA,pm_choice,1);
%
% compute expectation of the estimators
% (the factor (1/2)^norv is due to the probability measure on [-1,1])
     yp_err_sq_el = yp_err_sq_el;
     yq_err_sq_el = yq_err_sq_el;
     xq_err_sq_el = xq_err_sq_el;
%
     yp_err_el = sqrt(yp_err_sq_el); yp_err_est=norm(yp_err_el,2);
     fprintf('simplistic YP-estimate of energy error is %10.4e \n',yp_err_est)
%
     yq_err_el = sqrt(yq_err_sq_el); yq_err_est=norm(yq_err_el,2);
     fprintf('simplistic YQ-estimate of energy error is %10.4e \n',yq_err_est)
%
     xq_err_el = sqrt(xq_err_sq_el);
     fprintf('simplistic XQ-estimate of energy error is %10.4e \n',xq_err_est)
%
% compute total error estimate
     tot_err_est = sqrt(2*xq_err_est^2 + yp_err_est^2 + 2*yq_err_est^2);
     fprintf('simplistic estimate of energy error is %10.4e \n',tot_err_est)

  elseif qmethod ==2,
         fprintf('FAST a posteriori error estimation for Q2 element \n')
         [x_q2p1,y_q2p1,xy_q2p1,xyp_q2p1,eezz] = q2p1grid(x,y,xy,mv,bound);
         [hx,hy,eex] = eexgen(xy_q2p1,xyp_q2p1,mv,eezz); % check element numbering
         neumannb=[]; % Dirichlet problem
% debug
%P=1; indset=[0], noarv=0; norv=0;

% call stoch_diffpost_q2_ypx for the V_{YP}-estimator
         [yp_err_sq_el,yp_elerr,fe,ae] = stoch_diffpost_q2_ypx_old(xy,mv,mbound,x_gal,...
                                         eex,hx,hy,indset,P,norv,noarv,KL_DATA,0);

% call stoch_diffpost_q2_xq for the V_{XQ}-estimator
[xq_err_gal,xq_err_est,xq_err_sq_el] = stoch_diffpost_q2_xqx(xy,mv,bound,x_gal,...
                                        indset,polyd,norv,noarv,KL_DATA,pm_choice,1);

% call stoch_diffpost_q2_yqx for the V_{YQ}-estimator
[yq_err_sq_el,yq_elerr,fe,ae] = stoch_diffpost_q2_yqx(xy,mv,mbound,x_gal,...
                                eex,hx,hy,indset,polyd,norv,noarv,KL_DATA,pm_choice,0);

% (the factor (1/2)^norv is due to the probability measure on [-1,1])
         yp_err_sq_el = yp_err_sq_el;
         xq_err_sq_el = xq_err_sq_el;
         yq_err_sq_el = yq_err_sq_el;
%
         yp_err_el = sqrt(yp_err_sq_el); yp_err_est=norm(yp_err_el,2);
         fprintf('simplistic YP-estimate of energy error is %10.4e \n',yp_err_est)
         yq_err_el = sqrt(yq_err_sq_el); yq_err_est=norm(yq_err_el,2);
         fprintf('simplistic YQ-estimate of energy error is %10.4e \n',yq_err_est)
         xq_err_el = sqrt(xq_err_sq_el);
         fprintf('simplistic XQ-estimate of energy error is %10.4e \n',xq_err_est)

% compute total error estimate
         tot_err_est = sqrt(2*xq_err_est^2 + yp_err_est^2 + 2*yq_err_est^2);
         fprintf('simplistic estimate of energy error is %10.4e \n',tot_err_est)
  end

