function [err_sq_el,xx,fe,ade] = stoch_diffpost_q1_ypx(xy,ev,ebound,q1sol,...
          eex,hx,hy,indset,P,norv,noarv,KL_DATA,tout)
%STOCH_DIFFPOST_Q1_YPX computes YPX error estimator for stochastic Q1 solution
% V_{YP}-estimator: employs elementwise Q1-bubbles for the edge midpoints and centroid
%                   tensorised with the original set of polynomials,
%                   uses 'mean-based' bilinear form B_0
%   [zerr_sq_el,zelerr,zfe,zae] = stoch_diffpost_q1_ypx(xy,ev,ebound,x_gal,...
%                                  eex,hx,hy,indset,P,norv,noarv,KL_DATA,tout);
%   input
%          xy           vertex coordinate vector  
%          ev           element mapping matrix
%          ebound       element edge boundary matrix 
%          x_gal        Q1 solution vector
%          eex          element edge connectivity array
%          hx,hy        element mesh sizes
%          indset       index set of polynomial degrees
%          P            length of the index set
%          norv         number of random variables
%          noarv        the number of active random variables
%          KL_DATA      data related to KL-expansion
%          tout         output switch
%   output
%          zerr_sq_el   element error estimate
%          zelerr       elementwise error estimate
%          zfe          elementwise rhs vectors
%          zae          LDLT factorized element matrices
%
%   calls functions q1fluxjmps, gausspoints_oned, gausspoints_twod
%   SIFISS function: DJS; 29 May 2015
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

  fprintf('computing YP-error estimator...  \n')
  x=xy(:,1); y=xy(:,2);
  nel=length(ev(:,1));
  nn=5*P;
  err_sq_el = zeros(nel,1);
  xl_v=nan(nel,4); yl_v=nan(nel,4);

%
% construct the integration rule
  ngpt=3;
  [oneg,onew] = gausspoints_oned(ngpt);
  [s,t,wt] = gausspoints_twod(oneg,onew);
  nngpt=ngpt^2; ng=ngpt;
  t1=tic;
%
% inner loop over elements    
  for ivtx = 1:4
      xl_v(:,ivtx) = x(ev(:,ivtx));
      yl_v(:,ivtx) = y(ev(:,ivtx));
  end

% initialise local stochastic and deterministic matrices
%  ae = zeros(nel,nn,nn);
  ade = zeros(nel,5,5);
  elerr=zeros(nn,nel);
  adem = zeros(nel,4,4,4);

  xl_s = zeros(nel,4,4); yl_s = zeros(nel,4,4);
% compute local mid-edge coordinates
  xedge1(:,1)= 0.5*(xl_v(:,1)+xl_v(:,2)); yedge1(:,1)= 0.5*(yl_v(:,1)+yl_v(:,2));
  xedge2(:,1)= 0.5*(xl_v(:,2)+xl_v(:,3)); yedge2(:,1)= 0.5*(yl_v(:,2)+yl_v(:,3));
  xedge3(:,1)= 0.5*(xl_v(:,3)+xl_v(:,4)); yedge3(:,1)= 0.5*(yl_v(:,3)+yl_v(:,4));
  xedge4(:,1)= 0.5*(xl_v(:,4)+xl_v(:,1)); yedge4(:,1)= 0.5*(yl_v(:,4)+yl_v(:,1));
  xcentr(:,1)= 0.25*(xedge1(:)+xedge2(:)+xedge3(:)+xedge4(:));
  ycentr(:,1)= 0.25*(yedge1(:)+yedge2(:)+yedge3(:)+yedge4(:));
       if tout>0,
       elt=99;
       fprintf('\n\n test element check \n') 
       fprintf('vertex x-coordinates \n') 
       disp(xl_v(elt,:))
       fprintf('vertex y-coordinates \n') 
       disp(yl_v(elt,:))
%       fprintf('\n\n interior coordinates \n') 
%       fprintf('   xm(1) = %6.3e  ym(1) =  %6.3e\n',xedge1(elt), yedge1(elt))
%       fprintf('   xm(2) = %6.3e  ym(2) =  %6.3e\n',xedge2(elt), yedge2(elt))
%       fprintf('   xm(3) = %6.3e  ym(3) =  %6.3e\n',xedge3(elt), yedge3(elt))
%       fprintf('   xm(4) = %6.3e  ym(4) =  %6.3e\n',xedge4(elt), yedge4(elt))
%       fprintf('   xm(5) = %6.3e  ym(5) =  %6.3e\n',xcentr(elt), ycentr(elt))
       end 

% first 
        xl_s(:,1,1) = xl_v(:,1); yl_s(:,1,1) = yl_v(:,1);
        xl_s(:,1,2) = xedge1(:); yl_s(:,1,2) = yedge1(:);
        xl_s(:,1,3) = xcentr(:); yl_s(:,1,3) = ycentr(:);
        xl_s(:,1,4) = xedge4(:); yl_s(:,1,4) = yedge4(:);
% second     
        xl_s(:,2,1) = xedge1(:); yl_s(:,2,1) = yedge1(:);
        xl_s(:,2,2) = xl_v(:,2); yl_s(:,2,2) = yl_v(:,2);
        xl_s(:,2,3) = xedge2(:); yl_s(:,2,3) = yedge2(:);
        xl_s(:,2,4) = xcentr(:); yl_s(:,2,4) = ycentr(:);
% third 
        xl_s(:,3,1) = xcentr(:); yl_s(:,3,1) = ycentr(:);
        xl_s(:,3,2) = xedge2(:); yl_s(:,3,2) = yedge2(:);
        xl_s(:,3,3) = xl_v(:,3); yl_s(:,3,3) = yl_v(:,3);
        xl_s(:,3,4) = xedge3(:); yl_s(:,3,4) = yedge3(:);
% fourth 
        xl_s(:,4,1) = xedge4(:); yl_s(:,4,1) = yedge4(:);
        xl_s(:,4,2) = xcentr(:); yl_s(:,4,2) = ycentr(:);
        xl_s(:,4,3) = xedge3(:); yl_s(:,4,3) = yedge3(:);
        xl_s(:,4,4) = xl_v(:,4); yl_s(:,4,4) = yl_v(:,4);    

% inner-inner loop over subdivided elements
  for subelt=1:4
      for ivtx = 1:4
          xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
          yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
      end
% loop over Gauss points
      for igpt = 1:nngpt
          sigpt=s(igpt);
          tigpt=t(igpt);
          wght=wt(igpt);
% evaluate derivatives etc
          [jac_v,invjac_v,phi_v,dphidx_v,dphidy_v] = deriv(sigpt,tigpt,xl_m,yl_m);
          coeff_v = stoch_gauss_coeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
          for j = 1:4
              for i = 1:4
                  adem(:,subelt,i,j) = adem(:,subelt,i,j)+wght*coeff_v(:,1).*dphidx_v(:,i).*dphidx_v(:,j).*invjac_v(:);
                  adem(:,subelt,i,j) = adem(:,subelt,i,j)+wght*coeff_v(:,1).*dphidy_v(:,i).*dphidy_v(:,j).*invjac_v(:);
              end
          end
% end of Gauss point loop
      end
% debug
      if tout >0, 
        fprintf('\n\n test Q1 stiffness matrix \n') 
        fprintf('subelement : %g \n',subelt) 
        disp(squeeze(adem(elt,subelt,1:4,1:4))) 
      end
% end of subdivided element loop
  end

% manual assembly of subelement contributions
% first edge
        ade(:,1,1) = adem(:,1,2,2) + adem(:,2,1,1);
        ade(:,1,2) = adem(:,2,1,3);
        ade(:,1,4) = adem(:,1,2,4); 
        ade(:,1,5) = adem(:,1,2,3) + adem(:,2,1,4);
% second edge
        ade(:,2,2) = adem(:,2,3,3) + adem(:,3,2,2);
        ade(:,2,1) = adem(:,2,3,1);
        ade(:,2,3) = adem(:,3,2,4); 
        ade(:,2,5) = adem(:,2,3,4) + adem(:,3,2,1);
% third edge
        ade(:,3,3) = adem(:,3,4,4) + adem(:,4,3,3);
        ade(:,3,2) = adem(:,3,4,2);
        ade(:,3,4) = adem(:,4,3,1); 
        ade(:,3,5) = adem(:,3,4,1) + adem(:,4,3,2);
% fourth edge
        ade(:,4,4) = adem(:,4,1,1) + adem(:,1,4,4);
        ade(:,4,1) = adem(:,1,4,2);
        ade(:,4,3) = adem(:,4,1,3); 
        ade(:,4,5) = adem(:,4,1,2) + adem(:,1,4,3);
% centroid
        ade(:,5,5) = adem(:,1,3,3) + adem(:,2,4,4)+ adem(:,3,1,1) + adem(:,4,2,2);
        ade(:,5,1) = adem(:,1,3,2) + adem(:,2,4,1);
        ade(:,5,2) = adem(:,2,4,3) + adem(:,3,1,2);
        ade(:,5,3) = adem(:,3,1,4) + adem(:,4,2,3);
        ade(:,5,4) = adem(:,4,2,1) + adem(:,1,3,4);
% debug
        if tout>0,
        fprintf('\n\n test assembled 5x5 stiffness matrix \n') 
        fprintf('element : %g \n',elt) 
        disp(squeeze(ade(elt,1:5,1:5)))
        end


% compute local rhs-vectors
  res_int = stoch_intres_q1_yp(xy,xl_s,yl_s,ev,q1sol,indset,P,norv,noarv,KL_DATA);
  res_edge = stoch_edgeres_q1_ypx(xy,ev,ebound,q1sol,eex,hx,hy,indset,P,norv,noarv,KL_DATA);
  fprintf('yp estimate | res_int = %7.4e : res_edge = %7.4e\n',norm(res_int), norm(res_edge))
  fe = res_int - res_edge;
  % debug
        if tout>0,
        fprintf('right-hand side vector \n') 
        disp(squeeze(fe(elt,1:end)))  
        end
        
% solve for local estimate (sequential code)
%  for ielem = 1:nel
%      elerr(:,ielem) = squeeze(ae(ielem,1:nn,1:nn))\(fe(ielem,1:nn)'); 
%  end
%
% solve for local estimate (vectorized code)
% LDLT factorization 
% redefine nn
  nn=5;
  dd=zeros(nel,nn); rr=zeros(nel,nn);
  for kk=1:nn-1
      for pp=1:kk-1;
          rr(1:nel,pp)=dd(1:nel,pp).*ade(1:nel,kk,pp);
      end
      dd(1:nel,kk)=ade(1:nel,kk,kk);
      for pp=1:kk-1;
          dd(1:nel,kk)= dd(1:nel,kk)- ade(1:nel,kk,pp).*rr(1:nel,pp);
      end
      for ii=kk+1:nn
          for pp=1:kk-1;
              ade(1:nel,ii,kk)=ade(1:nel,ii,kk)-ade(1:nel,ii,pp).*rr(1:nel,pp);
          end
          ade(1:nel,ii,kk)=ade(1:nel,ii,kk)./dd(1:nel,kk);
      end
  end
  for pp=1:nn-1;
      rr(1:nel,pp)=dd(1:nel,pp).*ade(1:nel,nn,pp);
  end
  dd(1:nel,nn)=ade(1:nel,nn,nn);
  for pp=1:nn-1;
      dd(1:nel,nn)= dd(1:nel,nn)- ade(1:nel,nn,pp).*rr(1:nel,pp);
  end
% overwrite diagonal entries
  for kk=1:nn
      ade(1:nel,kk,kk)= dd(1:nel,kk);
  end
% forward-backward substitutions ...
% loop over the stochastic components
  elerr=zeros(nn*P,nel); 
  for nnp=1:P;
  fde=fe(:,(nnp-1)*5+1:nnp*5);
    if tout>0, fprintf('%g right-hand side\n',nnp),
    disp(squeeze(fde(elt,1:end))),end
  xdx = element_lusolve(ade,fde);
  elerr((nnp-1)*5+1:nnp*5,1:nel)=xdx';
  end
  xx=elerr';
%
  for svtx=1:nn*P
      err_sq_el(:) = err_sq_el(:) + fe(:,svtx) .* elerr(svtx,:)';
  end

  etime=toc(t1);
  fprintf('  Error estimation took %6.3e seconds\n',etime)

return

