function [energy] = refenergy_sg(norv,G_ref,K_ref,xy_ref,x_ref,polyd_ref)
%REFENERGY_SG     energy error using stochastic reference solution
%  [energy_sg] = refenergy_sg(noarv_ref,G_ref,K_ref,xy_ref,x_ref,polyd_ref);
%   input
%          norv          number of random variables | must be fixed
%          G_ref         reference G matrix structure | norv components
%          K_ref         reference K_nbc structure | norv components
%          xy_ref        fine grid coordinate vector
%          x_ref         fine grid solution
%          polyd_ref     fine problem polynomial degree 
%
%   output
%          energy         energy estimate
%
%   SIFISS function: DJS; 31 May 2015.
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

fprintf('\nComputing energy of SG reference solution\n')
fprintf('%g random variables | %g polynomial degree \n',norv,polyd_ref)

% reference grid
  indset_ref = cp_stoch_indset(polyd_ref,norv);
  xf=xy_ref(:,1); yf=xy_ref(:,2);
  nfnf=length(xf); nf=sqrt(nfnf);
  fprintf('reference grid is %3i by %3i | ',nf,nf)

% compute the energy error
b_ref = stoch_matvec(x_ref,G_ref,K_ref);
energy = sqrt(x_ref'*b_ref)/(sqrt(2.0e0)^norv);
fprintf('solution energy is %8.4e \n',energy)
return
