%STOCH_SQUARE_DIFF_ADAPT_X2 driver for the default adaptive algorithm
% to solve stochastic diffusion problem in square shaped domain
%
% automatic choice of refinement type
% refines according to the largest error reduction estimate for
% spatial refinement and for individual new indices
%
%   SIFISS scriptfile: DJS; 6 July 2017
% Copyright (c) 2015 A. Bespalov, C.E. Powell, D.J. Silvester
gohome, cd datafiles
save stoch_data.mat rd_type checkcode
clear

% set error estimate to Inf
  tot_err_est=Inf;

% set iteration count to zero
  iter=0;

% define geometry
  spde=1; domain=1;
  square_type=default('\n\nDomain: [0,1] or [-1,1] square (enter 1/2) (default is [0,1])',1);

% set the spatial approximation
  qmethod=default('Q1/Q2 approximation 1/2? (default Q1)',1);

if qmethod==1
    err_tol=default('desired error tolerance (2.5e-3)',2.5e-3);
elseif qmethod==2
    err_tol=default('desired error tolerance (2.5e-4)',2.5e-4);
end

% set up cell array for index sets
adaptmax=64;
index_iter=cell(1,adaptmax);

% start of while loop
  while tot_err_est > err_tol

        if iter==0 fprintf('\nGenerating coarse approximation\n')
% Define coarse approximation
%
% Generate coarse grid
           nc=default('Coarse grid parameter: 3 for underlying 8x8 grid (default is 16x16)',4);
           if nc<1, error('illegal parameter choice, try again.'), end
           grid_type=default('uniform/stretched grid (1/2) (default is uniform)',1);
           stoch_adapt_square_domain(square_type,grid_type,nc)
           load square_grid.mat
% find x_min, x_max, y_min, y_max
           x_min = min(xy(:,1)); x_max = max(xy(:,1));
           y_min = min(xy(:,2)); y_max = max(xy(:,2));
%
% limit the number of random variables
           norv=32;
% compute the KL-expansion data if needed
           gohome, cd datafiles
           load stoch_data  %% load rd_type checkcode
           if rd_type == 1 && norv > 0
% check that domain is symmetric w.r.t. the origin
              if x_min + x_max == 0 && y_min + y_max == 0
                 fprintf('setting up KL expansion data\n')
                 std_dev=default('standard deviation? (default 0.15)',1.5e-01);
                 correl_x=default('correlation length in x-direction? (default 1.0)',1.0e-00);
                 correl_y=default('correlation length in y-direction? (default 1.0)',1.0e-00);
                 KL_DATA = stoch_kl_eigens(x_max,y_max,std_dev,correl_x,correl_y,norv);
              else
                 error('the domain must be symmetric with respect to the origin!')
              end
           elseif rd_type == 2
                  fprintf('using Eigel synthetic random coefficient expansion\n')
                  KL_DATA.input = [1,1,0];
                  sdecay=default('slow/fast coefficient decay 1/0  (default slow)',1);
                  KL_DATA.decay = sdecay;
           elseif rd_type == 3
                  fprintf('using Powell synthetic random coefficient expansion\n')
                  KL_DATA.input = [1,1,0];
                  mu=default('mean coefficient (default is 2)',2);
                  KL_DATA.mu = mu;
                  ell=default('correlation length coefficient (default is 0.5)',0.5);
                  KL_DATA.ell = ell;
           else
              fprintf('using synthetic random coefficient expansion\n')
              KL_DATA.input = [1,1,0];
           end
% save the data related to KL-expansion
           save stoch_kl_data.mat '-struct' KL_DATA
%
% define coarse stochastic approximation
           noarv=default('Number of active random variables? (default is 1)',1);
           if noarv > norv, error('Oops.. need to increase parameter norv!'), end
           if noarv>0,
              polyd=default('Total polynomial degree? (default is 1)',1);
              [def_indset] = stoch_indset(polyd,noarv);
              def_indset = [def_indset,zeros(length(def_indset),norv-noarv)];
              indset=def_indset;
           else
% set up the default index set
              def_indset=[0]; 
              indset=default('Input index set of polynomial degrees? (accept default)',def_indset);
              noarv=length(indset(1,:));
              indset = [indset,zeros(length(indset(:,1)),norv-noarv)];
           end
           index_init=indset;
        else
% find maximum estimator and choose refinement type
           [~,refine_type]=max([yp_err_est,xq_err_vec(1)]);
           fprintf('\n\nIteration %i \n',iter)
% refining SGFEM space
           if refine_type==1
              if nc+1==10
                 fprintf('Terminated. Tolerance is not achieved.\n')
                 break
              end
              nc=nc+1;
              fprintf('Performing grid refinement, grid parameter is %i.\n', nc)
% Generate new grid
              stoch_adapt_square_domain(square_type,grid_type,nc)
              load square_grid.mat
           elseif refine_type==2
% debug
if checkcode,
fprintf('\nspatial error (YP) %g ',yp_err_est)
fprintf('\nindex set component errors (XQ)\n'),
   for jj=1:length(xq_err_vec(:,1))
   fprintf(' %3i   %8.5f \n', jj, xq_err_vec(jj)); end
   fprintf('\n')
end
%
% selecting new indices
              Qk=1;
              while  Qk~=length(xq_err_vec) & xq_err_vec(Qk+1) > yp_err_est %% fix DJS
              Qk=Qk+1;
              end
              [~,Q_noarv]=find(Q_indset(1:Qk,:));
              if max(Q_noarv)==norv
                 fprintf('Terminated: limit norv is achieved. Tolerance is not met.\n')
                 break
              end
              fprintf('Augmenting index set by \n')
              for fp=1:Qk
                  fprintf('%i ', Q_indset(fp,1:noarv+1)); fprintf('\n')
              end
              fprintf('\n')
              if iter <=adaptmax,index_iter{1,iter}=Q_indset(1:Qk,:); end
              indset=[indset;Q_indset(1:Qk,:)];
           elseif refine_type==4
              fprintf('Increasing the number of active random variables AND polynomial degree\n\n')
% define stochastic approximation
              noarv=noarv+1; polyd=polyd+1;
              if noarv > norv, error('Oops.. need to increase parameter norv!'), end
              [def_indset] = stoch_indset(polyd,noarv);
              def_indset = [def_indset,zeros(length(def_indset),norv-noarv)];
              indset=def_indset;
           end
        end

% sort the index set
        indset=sortrows(indset);
%
% find the length of the index set
        [P,norv] = size(indset);   % P=p+1 if norv=1
%
% find the number of active random variables
        noarv = norv - nnz(all(indset==0,1));
%
% find maximum (total) polynomial degree
        polyd=max(sum(indset,2));

% special case: indset = [0 0 ... 0]
        if P==1 && nnz(indset)==0
           noarv = 0; indset = zeros(1,norv);
        end

% reference grid switch
        savesol=0;
%        savesol=default('save results for reference 1/0 (yes/no)? (default no)',0);

% setup and solve the discrete system  
        fprintf('MINRES solution of the linear system ... \n')
        tol=1e-10; maxit=99;
        apost=0; stopit=0; prob_type='sdiff';
        if qmethod==1,
           [ev,ebound] = q1grid(xy,mv,bound,mbound);
           [Knbc,fnbc] = stoch_femq1_setup(xy,ev,indset,P,norv,noarv,KL_DATA);
        elseif qmethod ==2,  
           [x,y,xy] = q2grid(x,y,xy,mv,bound);
           [Knbc,fnbc] = stoch_femq2_setup(xy,mv,indset,P,norv,noarv,KL_DATA);
        else
            error('Oops.. the approximation is not available!')
        end
% generate the G-matrices
        G = stoch_gmatrices(indset,P,noarv,norv);
        [K,fnew,x_gal,dupl_intern] = stoch_imposebc(Knbc,fnbc,G,xy,bound,indset,P,norv,noarv);
        nint=length(dupl_intern)/P;
% extract the mean stiffness matrix
        Amean=K{1}; % imposed boundary conditions
        [LA,UA]=lu(Amean); 
        MA='m_sdiff'; aparams=struct('nint',nint,'L',LA,'U',UA);
        [xminres,resvec,emin, errpde, info]= ....
            stoch_est_minres(G,K,fnew,apost,stopit,maxit,tol,...
                             prob_type,MA,aparams);
        x_gal(dupl_intern)=xminres;
   
% plot convergence data
        figure(13)
         inx=0:info(4)-1;rx=9:9:info(4);
         if info(4)>4,
         semilogy(inx,sqrt(2)*resvec./emin,'-r');
         hold on
         semilogy(inx,resvec,'-b');
         hold off
         axis('square')
         xlabel('iteration number {\it k}')
         title('MINRES convergence history')
         h=legend('${\sqrt{2}/ \lambda_k^2}\> ||r_k||_{M_*}$','$||r_k||_{M_*}$');
         set(h,'Interpreter','latex')
        end
        fprintf('linear solver statistics:\n')
        fprintf('total ndof is %g\nminimum eigenvalue is %8.3f\n',length(x_gal),emin(end))
        fprintf('stochastic parameters:\n')
        fprintf('%g active random variables | polynomial degree is %2i\n',noarv,polyd)
        fprintf('\n')
   
% save solution data
        if savesol==1,
           xy_ref=xy; x_ref=x_gal; bound_ref=bound; norv_ref=norv; noarv_ref=noarv;
           dupl_intern_ref=dupl_intern; G_ref=G; K_ref=Knbc; P_ref=P; polyd_ref=polyd;
           indset_ref=indset;
           save stochrefsolution.mat xy_ref x_ref bound_ref norv_ref ...
                noarv_ref dupl_intern_ref G_ref K_ref P_ref polyd_ref indset_ref
        end

% compute variance
        nvtx=length(xy(:,1));
        var_sol=stoch_variance(x_gal,P);
        fprintf('\nComputed statistics ...\nmaximum mean value is %10.5e ',...
        max(x_gal(1:nvtx)) );
        fprintf('\nmaximum variance is %8.3e \n\n',max(var_sol));

% compute a posteriori error estimate
        stoch_adapt_diffpost_x

% original estimator code (3 componnets)
%  stoch_adapt_diffpost


% special case: indset = [0 0 ... 0]
        if P==1 && nnz(indset)==0
% increasing both noarv and polyd in the next iteration
           refine_type=4; iter=iter+1; continue
        end

% compute error reduction for (spatial) h-refinement
% the error reduction estimate is given by yp_err_est
% find the number of new degrees of freedom
        new_dof_yp=(length(x)-1)*(3*length(x)-1)*P;
        delta_yp=yp_err_est/new_dof_yp;

% output: yp_err_est, delta_yp
%        fprintf('error reduction for spatial refinement: %10.4e and %10.4e (per new dof)\n', yp_err_est, delta_yp)

% compute error reductions for all added 'neighbouring' indices
% the error reduction estimates for indices in Q_indset are given by (vector) xq_err_vec
% sort XQ-error reductions in descending order...
        [xq_err_vec,sort_index]=sortrows(xq_err_vec,-1);
% ... and sort Q_indset accordingly
        Q_indset=Q_indset(sort_index,:);

% output: Q_indset and xq_estimatrix
%        Q_indset
%        xq_err_vec

        iter=iter+1;
% debugging
%        if iter==3 break, end

%% save iteration data
% compute the energy error
b_ref = stoch_matvec(x_gal,G,Knbc);
energy = sqrt(x_gal'*b_ref);
fprintf('solution energy is %8.4e \n',energy)
energy_iter(iter,1)=energy; error_iter(iter,1)=tot_err_est;
dof_iter(iter,1)=length(x_gal);arv_iter(iter,1)=noarv;


% end of while loop
  end

  fprintf('\nfinal grid parameter: %i', nc)
  fprintf('\nfinal index set:\n')
  for k=1:P
      fprintf('%i ', indset(k,1:noarv)); fprintf('\n')
  end
  
 save adaptive_output.mat energy_iter error_iter dof_iter arv_iter index_iter index_init
 fprintf('\noutput data saved in adaptive_output.mat\n')

% plot expectation, variance and YP-error estimate
  if qmethod ==1,
     stoch_errplot_q1(x_gal,yp_err_el,ev,xy,x,y,P,noarv,111),
%     stoch_errplot_q1(x_gal,yq_err_el,ev,xy,x,y,P,noarv,113),
     stoch_errplot_q1(x_gal,xq_err_el,ev,xy,x,y,P,noarv,114),
%     fprintf('To test other detail space options .. run stoch_diffpost\n')
  elseif qmethod==2,
     stoch_errplot_q2(x_gal,mv,xy,x,y,P,noarv,112),
  end


