function stoch_errplotl_q1(sol,errdata,ev,xy,x,y,P,noarv,fig)
%STOCH_ERRPLOTL_Q1 plots stochastic solution data on L-shaped domain 
%   stoch_errplotl_q1(sol,errata,ev,xy,x,y,P,noarv,fig);
%   input
%          sol          nodal stochatic FE solution vector
%          errdata      element error vector
%          ev           element mapping matrix
%          xy           vertex coordinate vector  
%          x            vector of x-axis interpolation points
%          y            vector of y-axis interpolation points
%          P            length of the index set
%          noarv      the number of active random variables
%          fig          figure number
%
%   IFISS function: DJS; 5 July 2017.
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

%  fprintf('plotting expectation, variance and estimated errors... ')
  etype=rem(fig,10);
  nvtx = length(xy); nel=length(errdata);

% interpolate to a cartesian product mesh
  [X,Y] = meshgrid(x,y);

% plot the expectation of the solution
  e_xysol = griddata(xy(:,1),xy(:,2),sol(1:nvtx),X,Y);
  [II,JJ]=find(X<0 & Y<0); e_xysol(II,JJ)=nan;
  figure(fig)
  subplot(321),contour(X,Y,e_xysol,20),axis('square')
  axis('off'), ellx, title('Expectation of the FE Solution')
  subplot(322),mesh(X,Y,e_xysol),axis('square')
  view(330,30)

% compute variance of the solution
  var_sol=stoch_variance(sol,P);

% plot variance of the solution
  var_xysol = griddata(xy(:,1),xy(:,2),var_sol(1:nvtx),X,Y);
  var_xysol(II,JJ)=nan;
  subplot(323),contour(X,Y,var_xysol,20),axis('square')
  axis('off'), ellx, title('Variance of the FE Solution')
  subplot(324),mesh(X,Y,var_xysol),axis('square')
  view(330,30)

% plot a posteriori error estimate
  xx=xy(:,1); yy=xy(:,2);
%
% loop over elements
  for ielem = 1:nel
      xl = xx(ev(ielem,:));
      yl = yy(ev(ielem,:));
      xc(ielem,1) = 0.25*sum(xl);
      xc(ielem,2) = 0.25*sum(yl);
  end
%
% interpolate to a cartesian product mesh
  x=0.5*(x(1:end-1)+x(2:end));
  y=0.5*(y(1:end-1)+y(2:end));
  [X,Y]=meshgrid(x,y);
  err_xysol = griddata(xc(:,1),xc(:,2),errdata,X,Y);
  err_xysol(II,JJ)=nan;
  subplot(325),contour(X,Y,err_xysol,15),axis('square')
  axis('off'), ellx, 
  if etype==1,title('YP estimated error'),
  elseif etype==3,title('YQ estimated error'),
  elseif etype==4,title('XQ estimated error'), end
  subplot(326),mesh(X,Y,err_xysol),axis('square')
  view(330,30)

%  fprintf('done\n')
  return

