function a = stoch_specific_coeff(x,y,nel,norv,KL_DATA)
%STOCH_SPECIFIC_COEFF   synthetic stochastic diffusion coefficient
%   a = stoch_specific_coeff(x,y,nel,norv,KL_DATA)
%   input
%          x          x coordinate vector
%          y          y coordinate vector
%          nel        number of elements
%          norv       number of random variables
%          KL_DATA    data related to KL-expansion
%
% This expansion is discussed on p.393
% G. Lord, C.E. Powell, T. Shardlow,
% An Introduction to Computational Stochastic PDEs,
% Cambridge University Press, 2014.
%
%   SIFISS function: DJS; 1 April 2015.
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

% initialisation
  a=zeros(nel,norv+1);

% set a_0
mu = KL_DATA.mu;
a(:,1) = mu*ones(nel,1);

% set length scale parameter
ell = KL_DATA.ell;

% one dimensional functions
ind=[0:norv-1];
nu1=[0.5.*exp(-pi*ind.^2.*ell^2)]'; nu2=nu1;
% compute all possible product pairs of 1D eigs 
nu_prod=nu1*nu2';

% find 1D indices and compute the largest 2D eigs
for m=1:norv
[maxcol,indrow]=max(nu_prod);
[eigmax,jj]=max(maxcol);
ii=indrow(jj);
nu2D(m)=eigmax;
nu2Dpos(m,:)=[ii,jj];
nu_prod(ii,jj)=0;
end

% set a_m
for m=1:norv
k1=nu2Dpos(m,1); k2=nu2Dpos(m,2); sqeig=sqrt(nu2D(m));
% 1d eig indices run from 0 to norv-1
   if (k1==1)&(k2==1)
   a(:,m+1)=sqeig*ones;
   elseif (k1==1)&(k2~=1)
   a(:,m+1)=sqrt(2)*sqeig *cos((k2-1).*pi.*y);
   elseif (k1~=1)&(k2==1)
   a(:,m+1)=sqrt(2)*sqeig *cos((k1-1).*pi.*x);
   else 
   a(:,m+1)=2*sqeig *cos((k1-1).*pi.*x).*cos((k2-1).*pi.*y);
   end
end
return