function [dadx,dady] = stoch_specific_gradcoeff(x,y,nel,norv,KL_DATA)
%STOCH_SPECIFIC_GRADCOEFF   gradient of analytic KL stochastic diffusion coefficient 
%   [dadx,dady] = stoch_specific_gradcoeff(x,y,nel,norv,KL_DATA)
%   input
%          x            x coordinate vector
%          y            y coordinate vector 
%          nel          number of elements  
%          norv         number of random variables
%          KL_DATA      data related to KL-expansion
%
%   SIFISS function: DJS; 2 May 2013.
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

% initialisation
  dadx=zeros(nel,norv+1);
  dady=zeros(nel,norv+1);

% set da_0/dx and da_0/dy
  dadx(:,1)=zeros(nel,1);
  dady(:,1)=zeros(nel,1);

% get standard deviation
  sigma = KL_DATA.input(3);

% set da_m/dx and da_m/dy
  for m=1:norv
      lambda = KL_DATA.ev2d(m);
      nx = KL_DATA.ev2d_ind(m,1); ny = KL_DATA.ev2d_ind(m,2);
      omega_x = KL_DATA.ev_x(nx,2); alpha_x = KL_DATA.ev_x(nx,3);
      omega_y = KL_DATA.ev_y(ny,2); alpha_y = KL_DATA.ev_y(ny,3);
      type_x = KL_DATA.ef_indic(nx); type_y = KL_DATA.ef_indic(ny);
%
      fx = alpha_x * (((1 + type_x)/2) * cos(x*omega_x) + ((1 - type_x)/2) * sin(x*omega_x));
      fy = alpha_y * (((1 + type_y)/2) * cos(y*omega_y) + ((1 - type_y)/2) * sin(y*omega_y));
%
      dfxdx = alpha_x * omega_x * (((1 - type_x)/2) * cos(x*omega_x) - ((1 + type_x)/2) * sin(x*omega_x));
      dfydy = alpha_y * omega_y * (((1 - type_y)/2) * cos(y*omega_y) - ((1 + type_y)/2) * sin(y*omega_y));
%
      dadx(:,m+1) = sigma * sqrt(3.0e0) * sqrt(lambda) * dfxdx .* fy;
      dady(:,m+1) = sigma * sqrt(3.0e0) * sqrt(lambda) * fx .* dfydy;
  end
return

